SystemOrganization addCategory: #'RobotSoccer-Strategy'!
SystemOrganization addCategory: #'RobotSoccer-Tests'!
SystemOrganization addCategory: #'RobotSoccer-Viewer'!
SystemOrganization addCategory: #'RobotSoccer-Strategy-Example'!
SystemOrganization addCategory: #'RobotSoccer-Servers'!
SystemOrganization addCategory: #'RobotSoccer-Servers-Simulated'!

EllipseMorph subclass: #RSBallMorph
	instanceVariableNames: 'board myBall'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Viewer'!

!RSBallMorph commentStamp: 'Richo 9/17/2007 10:22' prior: 0!
Representa grficamente la pelota actual del partido.
Guarda una referencia a la instancia de Ball que est representando (en la variable "miBall") y es responsable de actualizar su propia posicin de acuerdo a lo que "miBall" indica.
Tambin responde al evento click abriendo un inspector de la instancia de Ball que representa, brindndonos informacin adicional durante el partido.
!

!RSBallMorph methodsFor: 'accessing' stamp: 'Richo 11/30/2007 22:43'!
board: anObject 
	board := anObject! !

!RSBallMorph methodsFor: 'event handling' stamp: 'Richo 6/23/2008 09:41'!
click: evt 
	myBall
		ifNil: [^ nil].
	myBall inspect! !

!RSBallMorph methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2007 11:31'!
defaultColor
^Color orange.! !

!RSBallMorph methodsFor: 'event handling' stamp: 'Richo 9/9/2007 20:49'!
handlesMouseDown: evt 
	^ true! !

!RSBallMorph methodsFor: 'as yet unclassified' stamp: 'Richo 6/22/2008 22:10'!
initialize
	super initialize.
	self extent: (RSBoardMorph centimetres: 4.27)
			* (1 @ 1)! !

!RSBallMorph methodsFor: 'stepping' stamp: 'Richo 6/23/2008 09:40'!
loadPosition: aPoint 
	"Posiciona el robot tomando como referencia el centro de la cancha"
	| diferenciaConElCentro difEnPixels posX posY posicionBall |
	posicionBall := aPoint.
	diferenciaConElCentro := posicionBall - RSBoardMorph centerFieldInInches.
	difEnPixels := RSBoardMorph inchesToPixels: diferenciaConElCentro.
	"Agrego lo siguiente porque el eje de coordenadas de squeak tiene el 0@0
	arriba y el simulador lo tiene abajo"
	posX := board bounds center x + difEnPixels x.
	posY := board bounds center y - difEnPixels y.
	self center: posX @ posY! !

!RSBallMorph methodsFor: 'event handling' stamp: 'Richo 9/9/2007 20:49'!
mouseDown: evt 
	evt hand waitForClicksOrDrag: self event: evt! !

!RSBallMorph methodsFor: 'accessing' stamp: 'Richo 11/30/2007 22:24'!
myBall: anObject 
	myBall := anObject! !

!RSBallMorph methodsFor: 'stepping' stamp: 'Richo 6/23/2008 09:42'!
step
	myBall
		ifNil: [^ nil].
	self loadPosition: myBall position! !

!RSBallMorph methodsFor: 'stepping' stamp: 'Richo 6/23/2008 09:29'!
stepTime
	^ board stepTime! !

Morph subclass: #RSBoardMorph
	instanceVariableNames: 'ball mousePositionMorph yellow blue game ballPredictions plugins'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Viewer'!

!RSBoardMorph commentStamp: 'Richo 6/22/2008 22:24' prior: 0!
"RSBoardMorph new openInWorld"

Esta clase representa la cancha y contiene todo lo necesario para mostrar el partido. Se encarga de conectar un environment y mostrar durante el partido las posiciones y rotacion de los robots y pelota.
Para esto, cuando se inicializa, adems de dibujar la cancha, crea las instancias grficas de los robots (propios y del oponente) y de la pelota.
Aunque no es el responsable de actualizar la posicin y rotacin de cada robot y pelota, s se encarga de hacer muchas de las conversiones entre las diferentes unidades de medicin (inches, centmetros y pixeles), as como de los ngulos del simulador y de Squeak. Estos mtodos de conversin estn implementados como mtodos de clase.
Por ltimo, este objeto tambin nos da informacin adicional: cuando pasamos el mouse dentro de sus lmites nos muestra (en los trminos del simulador, no de Squeak) las coordenadas del cursor dentro de la cancha.
!

!RSBoardMorph class methodsFor: 'constantes' stamp: 'Richo 12/1/2007 13:19'!
centerFieldInInches
	^ 50.11885 @ 41.8061! !

!RSBoardMorph class methodsFor: 'conversion' stamp: 'Richo 12/1/2007 13:23'!
centimetres: num 
	^ self centimetresToPixels: num! !

!RSBoardMorph class methodsFor: 'conversion' stamp: 'Richo 12/1/2007 13:22'!
centimetresToInches: num 
	^ num / self inchesToCentimetres! !

!RSBoardMorph class methodsFor: 'constantes' stamp: 'Richo 12/1/2007 13:21'!
centimetresToPixels
	^ 2.5! !

!RSBoardMorph class methodsFor: 'conversion' stamp: 'Richo 12/1/2007 13:22'!
centimetresToPixels: num 
	"Convierte de centimetros a pixeles"
	^ num * self centimetresToPixels! !

!RSBoardMorph class methodsFor: 'constantes' stamp: 'Richo 12/1/2007 13:22'!
inchesToCentimetres
	^ 2.54! !

!RSBoardMorph class methodsFor: 'conversion' stamp: 'Richo 12/1/2007 13:22'!
inchesToCentimetres: num 
	^ num * self inchesToCentimetres! !

!RSBoardMorph class methodsFor: 'conversion' stamp: 'Richo 9/7/2007 09:42'!
inchesToPixels: num 
^self centimetresToPixels: (self inchesToCentimetres: num)! !

!RSBoardMorph class methodsFor: 'conversion' stamp: 'Richo 12/1/2007 13:23'!
pixelsToCentimetres: num 
	"Convierte de pixeles a centimetros"
	^ num / self centimetresToPixels ! !

!RSBoardMorph class methodsFor: 'conversion' stamp: 'Richo 9/10/2007 11:08'!
pixelsToInches: num 
	^ self centimetresToInches: (self pixelsToCentimetres: num) ! !

!RSBoardMorph class methodsFor: 'conversion' stamp: 'Richo 9/9/2007 19:52'!
squeakAngle: simulatorAngle 
| angN dif |
angN_90 - simulatorAngle.
angN>180 ifTrue:[
	dif_ angN - 180.
	angN_-180 + dif.
	
	].
^angN! !

!RSBoardMorph methodsFor: 'bounds' stamp: 'Richo 12/1/2007 13:24'!
centralLineBounds
	| centro array |
	centro := self center.
	array := {centro x @ (centro y
					- (self class centimetres: 90)). centro x @ (centro y
					+ (self class centimetres: 90))}.
	^ array! !

!RSBoardMorph methodsFor: 'event handling' stamp: 'Richo 6/22/2008 22:10'!
convertMousePositionInCentimetres: aPoint 
	| posicion diferenciaConElCentro difEnCentimetros centroDeLaCancha posX posY |
	posicion := aPoint.
	diferenciaConElCentro := posicion - self center.
	difEnCentimetros := RSBoardMorph pixelsToCentimetres: diferenciaConElCentro.
	centroDeLaCancha := RSBoardMorph inchesToCentimetres: RSBoardMorph centerFieldInInches.
	posX := centroDeLaCancha x + difEnCentimetros x.
	posY := centroDeLaCancha y - difEnCentimetros y.
	^ (posX @ posY) truncated! !

!RSBoardMorph methodsFor: 'event handling' stamp: 'Richo 6/22/2008 22:10'!
convertMousePositionInInches: aPoint 
	| posicion diferenciaConElCentro centroDeLaCancha posX posY difEnInches |
	posicion := aPoint.
	diferenciaConElCentro := posicion - self center.
	difEnInches := RSBoardMorph pixelsToInches: diferenciaConElCentro.
	centroDeLaCancha := RSBoardMorph centerFieldInInches.
	posX := centroDeLaCancha x + difEnInches x.
	posY := centroDeLaCancha y - difEnInches y.
	^ (posX @ posY) truncated! !

!RSBoardMorph methodsFor: 'visual properties' stamp: 'Richo 9/3/2007 12:11'!
defaultColor
	^ Color black alpha: 0.7! !

!RSBoardMorph methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:13'!
disconnectAll
	game disconnectAll! !

!RSBoardMorph methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:13'!
disconnectRobotServer
	game disconnectRobotServer! !

!RSBoardMorph methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:13'!
disconnectTeam
	game disconnectTeam! !

!RSBoardMorph methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:11'!
disconnectVideoServer
game disconnectVideoServer! !

!RSBoardMorph methodsFor: 'drawing' stamp: 'Richo 11/17/2008 15:37'!
draw: vertices on: aCanvas 

	aCanvas
		drawPolygon: vertices
		color: Color transparent
		borderWidth: 2
		borderColor: self defaultColor negated! !

!RSBoardMorph methodsFor: 'drawing' stamp: 'Richo 11/17/2008 15:06'!
drawAreasOn: aCanvas
	self draw: self leftSmallAreaBounds on: aCanvas;
		 draw: self rightSmallAreaBounds on: aCanvas;
		 draw: self leftBigAreaBounds on: aCanvas;
		 draw: self rightBigAreaBounds on: aCanvas! !

!RSBoardMorph methodsFor: 'drawing' stamp: 'Richo 11/17/2008 15:06'!
drawBoundsOn: aCanvas 
		self draw: self fieldBounds on: aCanvas! !

!RSBoardMorph methodsFor: 'drawing' stamp: 'Richo 11/17/2008 14:48'!
drawCentralCircleOn: aCanvas 
	aCanvas
		frameOval: (self center
				- (self class centimetres: 25) corner: self center
					+ (self class centimetres: 25))
		width: 2
		color: self defaultColor negated! !

!RSBoardMorph methodsFor: 'drawing' stamp: 'Richo 11/17/2008 15:07'!
drawCentralLineOn: aCanvas 
self draw: self centralLineBounds on: aCanvas! !

!RSBoardMorph methodsFor: 'drawing' stamp: 'Richo 11/17/2008 15:08'!
drawFreeKickLinesOn: aCanvas 
	| centro inicio fin bordes |
	centro := self center.
	inicio := centro x
				- (self class centimetres: 110) @ (centro y
					- (self class centimetres: 90)).
	fin := centro x
				+ (self class centimetres: 110) @ (centro y
					+ (self class centimetres: 90)).
	bordes := inicio corner: fin.
	self draw: {bordes left
			+ (self class centimetres: 30) @ (bordes top
				+ (self class centimetres: 28)). bordes left
			+ (self class centimetres: 30) @ (bordes top
				+ (self class centimetres: 32))} on: aCanvas.
	self draw: {bordes left
			+ (self class centimetres: 30) @ (bordes bottom
				- (self class centimetres: 28)). bordes left
			+ (self class centimetres: 30) @ (bordes bottom
				- (self class centimetres: 32))} on: aCanvas.
	self draw: {bordes right
			- (self class centimetres: 30) @ (bordes top
				+ (self class centimetres: 28)). bordes right
			- (self class centimetres: 30) @ (bordes top
				+ (self class centimetres: 32))} on: aCanvas.
	self draw: {bordes right
			- (self class centimetres: 30) @ (bordes bottom
				- (self class centimetres: 28)). bordes right
			- (self class centimetres: 30) @ (bordes bottom
				- (self class centimetres: 32))} on: aCanvas.
	"Sigue..."
	self draw: {bordes left
			+ (self class centimetres: 80) @ (bordes top
				+ (self class centimetres: 28)). bordes left
			+ (self class centimetres: 80) @ (bordes top
				+ (self class centimetres: 32))} on: aCanvas.
	self draw: {bordes left
			+ (self class centimetres: 80) @ (bordes bottom
				- (self class centimetres: 28)). bordes left
			+ (self class centimetres: 80) @ (bordes bottom
				- (self class centimetres: 32))} on: aCanvas.
	self draw: {bordes right
			- (self class centimetres: 80) @ (bordes top
				+ (self class centimetres: 28)). bordes right
			- (self class centimetres: 80) @ (bordes top
				+ (self class centimetres: 32))} on: aCanvas.
	self draw: {bordes right
			- (self class centimetres: 80) @ (bordes bottom
				- (self class centimetres: 28)). bordes right
			- (self class centimetres: 80) @ (bordes bottom
				- (self class centimetres: 32))} on: aCanvas.! !

!RSBoardMorph methodsFor: 'drawing' stamp: 'Richo 11/17/2008 15:01'!
drawFreeKickPointsOn: aCanvas 
	| centro inicio fin bordes p |
	centro := self center.
	inicio := centro x
				- (self class centimetres: 110) @ (centro y
					- (self class centimetres: 90)).
	fin := centro x
				+ (self class centimetres: 110) @ (centro y
					+ (self class centimetres: 90)).
	bordes := inicio corner: fin.
	"******************Punto 1***********************"
	p := bordes left
				+ (self class centimetres: 55) @ (bordes top
					+ (self class centimetres: 30)).
	aCanvas
		frameOval: (p - 4 corner: p + 4)
		width: 2
		color: self defaultColor negated.
	"******************Punto 2***********************"
	p := bordes right
			- (self class centimetres: 55) @ (bordes top
				+ (self class centimetres: 30)).
	aCanvas
		frameOval: (p - 4 corner: p + 4)
		width: 2
		color: self defaultColor negated.
	"******************Punto 3***********************"
	p := bordes left
			+ (self class centimetres: 55) @ (bordes bottom
				- (self class centimetres: 30)).
	aCanvas
		frameOval: (p - 4 corner: p + 4)
		width: 2
		color: self defaultColor negated.
	"******************Punto 4***********************"
	p := bordes right
			- (self class centimetres: 55) @ (bordes bottom
				- (self class centimetres: 30)).
	aCanvas
		frameOval: (p - 4 corner: p + 4)
		width: 2
		color: self defaultColor negated.! !

!RSBoardMorph methodsFor: 'drawing' stamp: 'Richo 11/17/2008 15:07'!
drawGoalsOn: aCanvas 
self draw: self leftGoalBounds on: aCanvas;
	draw: self rightGoalBounds on: aCanvas! !

!RSBoardMorph methodsFor: 'drawing' stamp: 'Richo 11/17/2008 15:05'!
drawOn: aCanvas 
	aCanvas fillRectangle: self bounds color: self defaultColor.
	self drawBoundsOn: aCanvas;
		 drawGoalsOn: aCanvas;
		 drawAreasOn: aCanvas;
		 drawCentralLineOn: aCanvas;
		 drawCentralCircleOn: aCanvas;
		 drawPenaltyPointsOn: aCanvas;
		 drawFreeKickPointsOn: aCanvas;
		 drawFreeKickLinesOn: aCanvas! !

!RSBoardMorph methodsFor: 'drawing' stamp: 'Richo 11/17/2008 15:03'!
drawPenaltyPointsOn: aCanvas 
	| centro inicio fin bordes p |
	centro := self center.
	inicio := centro x
				- (self class centimetres: 110) @ (centro y
					- (self class centimetres: 90)).
	fin := centro x
				+ (self class centimetres: 110) @ (centro y
					+ (self class centimetres: 90)).
	bordes := inicio corner: fin.
	"******************Punto 1***********************"
	p := bordes left
				+ (self class centimetres: 37.5) @ centro y.
	aCanvas
		frameOval: (p - 4 corner: p + 4)
		width: 2
		color: self defaultColor negated.
	"******************Punto 2***********************"
	p := bordes right
				- (self class centimetres: 37.5) @ centro y.
	aCanvas
		frameOval: (p - 4 corner: p + 4)
		width: 2
		color: self defaultColor negated! !

!RSBoardMorph methodsFor: 'bounds' stamp: 'Richo 12/1/2007 13:26'!
fieldBounds
	| centro inicio fin bordes array distanciaEsquinas |
	centro := self center.
	inicio := centro x
				- (self class centimetres: 110) @ (centro y
					- (self class centimetres: 90)).
	fin := centro x
				+ (self class centimetres: 110) @ (centro y
					+ (self class centimetres: 90)).
	bordes := inicio corner: fin.
	distanciaEsquinas := 11.
	array := {bordes left @ (bordes top
					+ (self class centimetres: distanciaEsquinas)). bordes left
				+ (self class centimetres: distanciaEsquinas) @ bordes top. bordes right
				- (self class centimetres: distanciaEsquinas) @ bordes top. bordes right @ (bordes top
					+ (self class centimetres: distanciaEsquinas)). bordes right @ (bordes bottom
					- (self class centimetres: distanciaEsquinas)). bordes right
				- (self class centimetres: distanciaEsquinas) @ bordes bottom. bordes left
				+ (self class centimetres: distanciaEsquinas) @ bordes bottom. bordes left @ (bordes bottom
					- (self class centimetres: distanciaEsquinas)). bordes left @ (bordes top
					+ (self class centimetres: distanciaEsquinas))}.
	^ array! !

!RSBoardMorph methodsFor: 'accessing' stamp: 'Richo 7/12/2008 22:21'!
game
^game! !

!RSBoardMorph methodsFor: 'event handling' stamp: 'Richo 12/1/2007 13:15'!
handleListenEvent: evt 
	| pos |
	((evt isMouse
				and: [evt isMove])
			and: [evt isMouseDown not])
		ifFalse: [^ self].
	pos := evt hand position.
	self refreshMousePosition: pos! !

!RSBoardMorph methodsFor: 'initialization' stamp: 'Richo 11/17/2008 15:09'!
initialize
	super initialize.
	self
		extent: (self class centimetres: 270 @ 200);
		 cornerStyle: #rounded.
	self initializeMousePositionMorph.
	self
		on: #mouseEnter
		send: #mouseEnter:
		to: self;
		
		on: #mouseLeave
		send: #mouseLeave:
		to: self.

	self initializeTeams.
	self initializeBall.
	self loadInitialPositions.
	game := RSGame new! !

!RSBoardMorph methodsFor: 'initialization' stamp: 'Richo 7/7/2008 11:56'!
initializeBall
	ball := RSBallMorph new board: self.

	self addMorph: ball! !

!RSBoardMorph methodsFor: 'initialization' stamp: 'Richo 9/10/2007 10:50'!
initializeMousePositionMorph
	mousePositionMorph := '' asMorph.
	mousePositionMorph bottom: self bottom - 4;
		 left: self left + 8;
		 color: self defaultColor negated.
	self addMorph: mousePositionMorph! !

!RSBoardMorph methodsFor: 'initialization' stamp: 'Richo 7/12/2008 22:15'!
initializeTeams
	blue := Array new: 5.
	blue
		at: 1
		put: (RSRobotMorph teamColor: Color blue playerColor: Color red).
	blue
		at: 2
		put: (RSRobotMorph
				teamColor: Color blue
				playerColor: (Color red mixed: 0.5 with: Color blue)).
	blue
		at: 3
		put: (RSRobotMorph teamColor: Color blue playerColor: Color red muchLighter).
	blue
		at: 4
		put: (RSRobotMorph teamColor: Color blue playerColor: Color green).
	blue
		at: 5
		put: (RSRobotMorph teamColor: Color blue playerColor: Color blue muchLighter).
	yellow := Array new: 5.
	yellow
		at: 1
		put: (RSRobotMorph teamColor: Color yellow playerColor: Color red).
	yellow
		at: 2
		put: (RSRobotMorph
				teamColor: Color yellow
				playerColor: (Color red mixed: 0.5 with: Color blue)).
	yellow
		at: 3
		put: (RSRobotMorph teamColor: Color yellow playerColor: Color red muchLighter).
	yellow
		at: 4
		put: (RSRobotMorph teamColor: Color yellow playerColor: Color green).
	yellow
		at: 5
		put: (RSRobotMorph teamColor: Color yellow playerColor: Color blue muchLighter).
	blue
		do: [:r | 
			r board: self.
			self addMorph: r].
	yellow
		do: [:r | 
			r board: self.
			self addMorph: r]! !

!RSBoardMorph methodsFor: 'waiting connection' stamp: 'Richo 11/17/2008 16:18'!
isWaitingProcessRunning
"Se fija si hay un proceso de espera de conexin activo"
	^ (Process allSubInstances
		reject: [:each | each isTerminated | (each name ~= self waitingProcessName)]) notEmpty! !

!RSBoardMorph methodsFor: 'bounds' stamp: 'Richo 12/1/2007 13:26'!
leftBigAreaBounds
	| centro inicio fin bordes array |
	centro := self center.
	inicio := centro x
				- (self class centimetres: 110) @ (centro y
					- (self class centimetres: 90)).
	fin := centro x
				+ (self class centimetres: 110) @ (centro y
					+ (self class centimetres: 90)).
	bordes := inicio corner: fin.
	array := {bordes left @ (centro y
					- (self class centimetres: 40)). bordes left
				+ (self class centimetres: 35) @ (centro y
					- (self class centimetres: 40)). bordes left
				+ (self class centimetres: 35) @ (centro y
					+ (self class centimetres: 40)). bordes left @ (centro y
					+ (self class centimetres: 40))}.
	^ array! !

!RSBoardMorph methodsFor: 'bounds' stamp: 'Richo 12/1/2007 13:26'!
leftGoalBounds
	| centro inicio fin bordes array |
	centro := self center.
	inicio := centro x
				- (self class centimetres: 110) @ (centro y
					- (self class centimetres: 90)).
	fin := centro x
				+ (self class centimetres: 110) @ (centro y
					+ (self class centimetres: 90)).
	bordes := inicio corner: fin.
	array := {bordes left @ (centro y
					- (self class centimetres: 20)). bordes left
				- (self class centimetres: 15) @ (centro y
					- (self class centimetres: 20)). bordes left
				- (self class centimetres: 15) @ (centro y
					+ (self class centimetres: 20)). bordes left @ (centro y
					+ (self class centimetres: 20))}.
	^ array! !

!RSBoardMorph methodsFor: 'bounds' stamp: 'Richo 12/1/2007 13:26'!
leftSmallAreaBounds
	| centro inicio fin bordes array |
	centro := self center.
	inicio := centro x
				- (self class centimetres: 110) @ (centro y
					- (self class centimetres: 90)).
	fin := centro x
				+ (self class centimetres: 110) @ (centro y
					+ (self class centimetres: 90)).
	bordes := inicio corner: fin.
	array := {bordes left @ (centro y
					- (self class centimetres: 25)). bordes left
				+ (self class centimetres: 15) @ (centro y
					- (self class centimetres: 25)). bordes left
				+ (self class centimetres: 15) @ (centro y
					+ (self class centimetres: 25)). bordes left @ (centro y
					+ (self class centimetres: 25))}.
	^ array! !

!RSBoardMorph methodsFor: 'initialization' stamp: 'Richo 7/9/2008 22:25'!
loadInitialPositions
	| aux |
	ball center: self center.
	aux := 0.
	1
		to: 5
		do: [:i | 
			aux := aux + 25.
			(yellow at: i)
				center: self center x - aux @ self center y.
			(blue at: i)
				center: self center x + aux @ self center y]! !

!RSBoardMorph methodsFor: 'event handling' stamp: 'Richo 9/10/2007 10:31'!
mouseEnter: evt
	evt hand addMouseListener: self! !

!RSBoardMorph methodsFor: 'event handling' stamp: 'Richo 12/1/2007 13:15'!
mouseLeave: evt 
	evt hand removeMouseListener: self.
	self refreshMousePosition: ''! !

!RSBoardMorph methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:06'!
pause
game 

pause! !

!RSBoardMorph methodsFor: 'accessing' stamp: 'Richo 7/7/2008 12:26'!
plugins: aCollection
plugins := aCollection! !

!RSBoardMorph methodsFor: 'event handling' stamp: 'Richo 7/14/2008 12:10'!
refreshMousePosition: aPoint 
	| mouseInches mouseCentimetres |
	aPoint = ''
		ifTrue: [mousePositionMorph contents: ''.
			^ self].
	mouseInches := self convertMousePositionInInches: aPoint.
	mouseCentimetres := self convertMousePositionInCentimetres: aPoint.
	mousePositionMorph contents: 'Mouse coordinates: ' , mouseInches asString , ' (' , mouseCentimetres asString , ' cm)'! !

!RSBoardMorph methodsFor: 'bounds' stamp: 'Richo 12/1/2007 13:27'!
rightBigAreaBounds
	| centro inicio fin bordes array |
	centro := self center.
	inicio := centro x
				- (self class centimetres: 110) @ (centro y
					- (self class centimetres: 90)).
	fin := centro x
				+ (self class centimetres: 110) @ (centro y
					+ (self class centimetres: 90)).
	bordes := inicio corner: fin.
	array := {bordes right @ (centro y
					- (self class centimetres: 40)). bordes right
				- (self class centimetres: 35) @ (centro y
					- (self class centimetres: 40)). bordes right
				- (self class centimetres: 35) @ (centro y
					+ (self class centimetres: 40)). bordes right @ (centro y
					+ (self class centimetres: 40))}.
	^ array! !

!RSBoardMorph methodsFor: 'bounds' stamp: 'Richo 12/1/2007 13:27'!
rightGoalBounds
	| centro inicio fin bordes array |
	centro := self center.
	inicio := centro x
				- (self class centimetres: 110) @ (centro y
					- (self class centimetres: 90)).
	fin := centro x
				+ (self class centimetres: 110) @ (centro y
					+ (self class centimetres: 90)).
	bordes := inicio corner: fin.
	array := {bordes right @ (centro y
					- (self class centimetres: 20)). bordes right
				+ (self class centimetres: 15) @ (centro y
					- (self class centimetres: 20)). bordes right
				+ (self class centimetres: 15) @ (centro y
					+ (self class centimetres: 20)). bordes right @ (centro y
					+ (self class centimetres: 20))}.
	^ array! !

!RSBoardMorph methodsFor: 'bounds' stamp: 'Richo 12/1/2007 13:27'!
rightSmallAreaBounds
	| centro inicio fin bordes array |
	centro := self center.
	inicio := centro x
				- (self class centimetres: 110) @ (centro y
					- (self class centimetres: 90)).
	fin := centro x
				+ (self class centimetres: 110) @ (centro y
					+ (self class centimetres: 90)).
	bordes := inicio corner: fin.
	array := {bordes right @ (centro y
					- (self class centimetres: 25)). bordes right
				- (self class centimetres: 15) @ (centro y
					- (self class centimetres: 25)). bordes right
				- (self class centimetres: 15) @ (centro y
					+ (self class centimetres: 25)). bordes right @ (centro y
					+ (self class centimetres: 25))}.
	^ array! !

!RSBoardMorph methodsFor: 'actions' stamp: 'Richo 7/14/2008 12:25'!
setRobotsAndBall
	game rsEnvironment isYellowTeam
		ifTrue: [1
				to: game numberOfRobots
				do: [:i | 
					(yellow at: i)
						myRobot: (game rsEnvironment homeAt: i).
					(blue at: i)
						myRobot: (game rsEnvironment opponentAt: i)]]
		ifFalse: [1
				to: game numberOfRobots
				do: [:i | 
					(blue at: i)
						myRobot: (game rsEnvironment homeAt: i).
					(yellow at: i)
						myRobot: (game rsEnvironment opponentAt: i)]].
	ball myBall: game rsEnvironment currentBall! !

!RSBoardMorph methodsFor: 'pruebas' stamp: 'Richo 7/3/2008 10:58'!
showMouseAngle: aPoint 
	| robot mouseInches |
	aPoint = ''
		ifTrue: [mousePositionMorph contents: ''.
			^ self].
	robot := RSRobot new.
	robot position: self class centerFieldInInches.
	mouseInches := self convertMousePositionInInches: aPoint.
	mousePositionMorph contents: 'Angulo del mouse con respecto al centro (en grados): ' , (robot getAngle: mouseInches) asString! !

!RSBoardMorph methodsFor: 'actions' stamp: 'Richo 11/17/2008 16:28'!
start
	game start.
	self isWaitingProcessRunning
		ifFalse: [self startWaitingProcess]
! !

!RSBoardMorph methodsFor: 'waiting connection' stamp: 'Richo 11/17/2008 16:31'!
startWaitingProcess
"Se hace un proceso que muestra un mensaje de espera de conexin para no congelar Squeak mientras se espera una conexin nueva. Es necesario no mandar el mensaje #setRobotsAndBall hasta que se conecte porque crashea"
	[| strMorph | 
	strMorph := self waitingForConnectionStringMorph.
	self addMorph: strMorph.
	[(Delay forMilliseconds: 100) wait.
	game rsEnvironment isYellowTeam isNil] whileTrue.
	strMorph delete.
	self setRobotsAndBall;startStepping]
		forkNamed: self waitingProcessName! !

!RSBoardMorph methodsFor: 'stepping' stamp: 'Richo 7/9/2008 22:24'!
step
	plugins
		do: [:each | each stepIfActive]! !

!RSBoardMorph methodsFor: 'stepping' stamp: 'Richo 11/17/2008 15:25'!
stepTime
	"Slo porque los robots y la pelota referencian ac"
	^ 100! !

!RSBoardMorph methodsFor: 'actions' stamp: 'Richo 11/17/2008 16:05'!
stop
	game stop.
	self terminateWaitingProcess! !

!RSBoardMorph methodsFor: 'waiting connection' stamp: 'Richo 11/17/2008 16:18'!
terminateWaitingProcess
"Termina todos los procesos de espera de conexin que esten activos"
	(Process allSubInstances
		reject: [:each | each isTerminated | (each name ~= self waitingProcessName)])
		do: [:each | each terminate]! !

!RSBoardMorph methodsFor: 'waiting connection' stamp: 'Richo 11/17/2008 16:26'!
waitingForConnectionStringMorph
	^ 'Waiting for connection ...' asMorph fontName: 'Accuny' size: 30;
		 emphasis: 0;
		 color: Color red;
		 center: self center x @ (self center y - 30)! !

!RSBoardMorph methodsFor: 'waiting connection' stamp: 'Richo 11/17/2008 16:08'!
waitingProcessName
	^ 'RobotSoccer - waiting for connection...'! !

AlignmentMorph subclass: #RSViewerMorph
	instanceVariableNames: 'board videoServerStringMorph robotServerStringMorph teamStringMorph numberOfRobotsStringMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Viewer'!

!RSViewerMorph commentStamp: 'Richo 6/22/2008 22:24' prior: 0!
"RSViewerMorph new openInWorld"

Esta clase representa la interfaz principal del simulador, y es la primer frontera de interaccin con el usuario.
Cuando se crea, inicializa un RSBoardMorph (que representa la cancha) y lo guarda en la variable de instancia "board". Tambin inicializa los botones "Conectar", "Start", "Desconectar" y "Salir" por medio de los cuales interactuamos con el simulador.
El botn "Conectar" ejecuta el mtodo #elegirEnvironment. Este mtodo crea un array con Environment y todas sus subclases, y levanta un men que nos permite elegir. Una vez que hemos elegido le manda el mensaje #conectar: al "board" (un RobotSoccerMorph) mandando como parmetro el environment que elegimos.
El botn "Start" simplemente manda el mensaje #start al "board", el cual manda el mensaje #start al environment (si est conectado).
El botn "Desconectar" ejecuta el mtodo #desconectar que lo que hace es mandar el mensaje #desconectar al "board", eliminar todos los submorphs, y volver a crearlos. De esta forma, acta como un "reset" que deja todo preparado para un nuevo partido.
El botn "Salir" elimina el objeto.


!

!RSViewerMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 3/7/2008 08:06'!
descriptionForPartsBin
	^ self
		partName: 'Robot Soccer Viewer'
		categories: #('Robot Soccer' )
		documentation: ''! !

!RSViewerMorph methodsFor: 'string morphs' stamp: 'Richo 7/14/2008 12:03'!
actualizeStringMorphs
	robotServerStringMorph contents: self boardGameRobotServer asString.
	teamStringMorph contents: self boardGameTeam asString.
	videoServerStringMorph contents: self boardGameVideoServer asString.
	numberOfRobotsStringMorph contents: self boardGameNumberOfRobots asString.! !

!RSViewerMorph methodsFor: 'initialize-release' stamp: 'Richo 7/9/2008 22:17'!
addPluginList
	| plugins |
	plugins := RSViewerPlugin allSubclasses asArray
				collect: [:each | each board: board].
plugins
				do: [:each | self addMorphBack: each menu].
			board plugins: plugins! !

!RSViewerMorph methodsFor: 'accessing' stamp: 'Richo 7/14/2008 12:03'!
boardGameNumberOfRobots
^board game numberOfRobots! !

!RSViewerMorph methodsFor: 'accessing' stamp: 'Richo 7/14/2008 12:03'!
boardGameNumberOfRobots: aSmallInteger
board game numberOfRobots: aSmallInteger! !

!RSViewerMorph methodsFor: 'accessing' stamp: 'Richo 7/14/2008 10:13'!
boardGameRobotServer
^board game robotServer! !

!RSViewerMorph methodsFor: 'accessing' stamp: 'Richo 7/14/2008 10:14'!
boardGameRobotServer: aRobotServer
board game robotServer:aRobotServer! !

!RSViewerMorph methodsFor: 'accessing' stamp: 'Richo 7/14/2008 10:13'!
boardGameTeam
	^ board game team! !

!RSViewerMorph methodsFor: 'accessing' stamp: 'Richo 7/14/2008 10:14'!
boardGameTeam: aTeam 
board game team: aTeam! !

!RSViewerMorph methodsFor: 'accessing' stamp: 'Richo 7/14/2008 10:13'!
boardGameVideoServer
	^ board game videoServer! !

!RSViewerMorph methodsFor: 'accessing' stamp: 'Richo 7/14/2008 10:14'!
boardGameVideoServer: aVideoServer
board game videoServer: aVideoServer! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/14/2008 12:05'!
changeNumberOfRobots
	| n |
	n := (FillInTheBlank request: 'Number of robots:' initialAnswer: self boardGameNumberOfRobots asString) asInteger.
	self boardGameNumberOfRobots: n.
	self actualizeStringMorphs! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/14/2008 10:26'!
chooseRobotServer
	| choice robotServers strRobotServers |
	robotServers := RSRobotServer allSubclasses asArray.
	strRobotServers := robotServers
				collect: [:each | '  ' , each asString , '  '].
	choice := PopUpMenu withCaption: 'RobotServers:' chooseFrom: strRobotServers.
	choice = 0
		ifTrue: [^ nil].
	^robotServers at: choice! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/14/2008 11:48'!
chooseTeam
	| teams strTeams choice |
		
	teams := RSTeam allSubclasses asArray.
	strTeams := teams
				collect: [:each | '  ' , each asString , '  '].
	choice := PopUpMenu withCaption: 'Teams:' chooseFrom: strTeams.
	choice = 0
		ifTrue: [^ nil].
	^ teams at: choice! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/14/2008 10:26'!
chooseVideoServer
	| choice videoServers strVideoServers |
	videoServers := RSVideoServer allSubclasses asArray.
	strVideoServers := videoServers
				collect: [:each | '  ' , each asString , '  '].
	choice := PopUpMenu withCaption: 'VideoServers:' chooseFrom: strVideoServers.
	choice = 0
		ifTrue: [^ nil].
	^videoServers at: choice! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/14/2008 10:24'!
connectRobotServer
| robotServer |
robotServer := self chooseRobotServer.
robotServer ifNotNil:[
	self boardGameRobotServer: robotServer fromUser.
	self actualizeStringMorphs
	]! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/14/2008 11:48'!
connectTeam

	| team |
"	self boardGameRobotServer
		ifNil: [^ self robotServerError]."
	team := self chooseTeam.
	team
		ifNotNil: [self boardGameTeam: team new.
			self actualizeStringMorphs]! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/14/2008 10:25'!
connectVideoServer
	| videoServer |
	videoServer := self chooseVideoServer.
videoServer
		ifNotNil: [self boardGameVideoServer: videoServer fromUser.
			self actualizeStringMorphs]! !

!RSViewerMorph methodsFor: 'visual properties' stamp: 'Richo 9/16/2007 13:38'!
defaultColor
	^ (Color
		r: 0.817
		g: 0.864
		b: 0.797)
		mixed: 0.75
		with: Color blue muchDarker! !

!RSViewerMorph methodsFor: 'initialize-release' stamp: 'Richo 7/10/2008 12:13'!
delete
	self disconnectAll.
	super delete! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/14/2008 12:16'!
disconnectAll
self disconnectVideoServer.
			self disconnectRobotServer.
			self disconnectTeam! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:18'!
disconnectRobotServer

		board disconnectRobotServer. self actualizeStringMorphs! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:19'!
disconnectTeam
	board disconnectTeam.
	self actualizeStringMorphs! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:19'!
disconnectVideoServer
board disconnectVideoServer.
			self actualizeStringMorphs! !

!RSViewerMorph methodsFor: 'initialize-release' stamp: 'Richo 9/16/2007 12:57'!
initialize
	super initialize.
	self borderWidth: 2;
		 borderColor: #raised;
		 hResizing: #shrinkWrap;
		 vResizing: #shrinkWrap;
		 listDirection: #topToBottom;
		 cornerStyle: #rounded.
	self initializeSubmorphs.! !

!RSViewerMorph methodsFor: 'initialize-release' stamp: 'Richo 7/16/2008 10:07'!
initializeSubmorphs
board := RSBoardMorph new.
	self addMorphBack: self makeExitPanel;
		 addMorphBack: self makeConnectionAndControlPanel;
		 addMorphBack: board.
	self addPluginList! !

!RSViewerMorph methodsFor: 'control panel' stamp: 'Richo 7/10/2008 12:04'!
makeConnectionAndControlPanel
	^ AlignmentMorph newRow color: Color transparent;
		 addMorphBack: self makeConnectionPanel;
		 addMorphBack: self makeControlPanel! !

!RSViewerMorph methodsFor: 'control panel' stamp: 'Richo 7/10/2008 11:56'!
makeConnectionPanel
	^ AlignmentMorph newColumn color: Color transparent;
		 addMorphBack: self makeVideoServerControlPanel;
		 addMorphBack: self makeRobotServerControlPanel;
		 addMorphBack: self makeTeamControlPanel! !

!RSViewerMorph methodsFor: 'control panel' stamp: 'Richo 7/14/2008 12:29'!
makeControlPanel
	^ AlignmentMorph newColumn color: Color transparent;
		 addMorphBack: self makeStartPauseStopPanel;
		 addMorphBack: AlignmentMorph newVariableTransparentSpacer;
		 addMorphBack: self makeNumberOfRobotsPanel! !

!RSViewerMorph methodsFor: 'control panel' stamp: 'Richo 7/10/2008 12:07'!
makeExitPanel
	^ AlignmentMorph newRow color: Color transparent;
		 cornerStyle: #rounded;
		 borderWidth: 0;
		 layoutInset: 3;
		 vResizing: #shrinkWrap;
		 wrapCentering: #center;
		 addMorphBack: ('RobotSoccerViewer' asMorph fontName: 'Accuny' size: 22;
			 emphasis: 4);
		 addMorphBack: AlignmentMorph newVariableTransparentSpacer;
		 addMorphBack: (SimpleButtonMorph new label: '  X  ';
			
			color: (Color red mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 borderWidth: 2;
			 actionSelector: #delete;
			 target: self)! !

!RSViewerMorph methodsFor: 'control panel' stamp: 'Richo 7/16/2008 10:08'!
makeNumberOfRobotsPanel
	numberOfRobotsStringMorph := self boardGameNumberOfRobots asMorph emphasis: 1.
	^ AlignmentMorph newRow color: Color transparent;
		 cornerStyle: #rounded;
		 borderWidth: 0;
		 layoutInset: 3;
		 vResizing: #shrinkWrap;
		 wrapCentering: #center;
		 addMorphBack: AlignmentMorph newVariableTransparentSpacer;
		 addMorphBack: 'Number of robots: ' asMorph;
		 addMorphBack: numberOfRobotsStringMorph;
		 addMorphBack: ' ' asMorph;
		 addMorphBack: (SimpleButtonMorph new label: '...';
			 color: self defaultColor;
			 borderColor: #raised;
			 borderWidth: 2;
			 actionSelector: #changeNumberOfRobots;
			 target: self)! !

!RSViewerMorph methodsFor: 'control panel' stamp: 'Richo 7/16/2008 10:08'!
makeRobotServerControlPanel
	robotServerStringMorph := self boardGameRobotServer  asString asMorph.
	^ AlignmentMorph newRow color: Color transparent;
		 addMorphBack: (SimpleButtonMorph new label: '  RobotServer ...  ';
			
			color: (Color blue mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 borderWidth: 2;
			 actionSelector: #connectRobotServer;
			 target: self);
		 addMorphBack: (SimpleButtonMorph new label: '  Disconnect  ';
			
			color: (Color red mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 borderWidth: 2;
			 actionSelector: #disconnectRobotServer;
			 target: self);
		 addMorphBack: ' --> ' asMorph;
		 addMorphBack: robotServerStringMorph;
		 addMorphBack: AlignmentMorph newVariableTransparentSpacer! !

!RSViewerMorph methodsFor: 'control panel' stamp: 'Richo 7/14/2008 12:02'!
makeStartPauseStopPanel
	^ AlignmentMorph newRow color: Color transparent;
		 cornerStyle: #rounded;
		 borderWidth: 0;
		 layoutInset: 3;
		 vResizing: #shrinkWrap;
		 wrapCentering: #center;
				 addMorphBack: AlignmentMorph newVariableTransparentSpacer ;
		 addMorphBack: (SimpleButtonMorph new label: '  Start  ';
			
			color: (Color green mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 borderWidth: 2;
			 actionSelector: #start;
			 target: self);
		 addMorphBack: (SimpleButtonMorph new label: '  Pause  ';
			
			color: (Color blue mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 borderWidth: 2;
			 actionSelector: #pause;
			 target: self);
		 addMorphBack: (SimpleButtonMorph new label: '  Stop  ';
			
			color: (Color red mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 borderWidth: 2;
			 actionSelector: #stop;
			 target: self)! !

!RSViewerMorph methodsFor: 'control panel' stamp: 'Richo 7/16/2008 10:08'!
makeTeamControlPanel
	teamStringMorph := self boardGameTeam  asString asMorph.
	^ AlignmentMorph newRow color: Color transparent;
		 addMorphBack: (SimpleButtonMorph new label: '  Team ...  ';
			
			color: (Color blue mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 borderWidth: 2;
			 actionSelector: #connectTeam;
			 target: self);
		 addMorphBack: (SimpleButtonMorph new label: '  Disconnect  ';
			
			color: (Color red mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 borderWidth: 2;
			 actionSelector: #disconnectTeam;
			 target: self);
		 addMorphBack: ' --> ' asMorph;
		 addMorphBack: teamStringMorph;
		 addMorphBack: AlignmentMorph newVariableTransparentSpacer! !

!RSViewerMorph methodsFor: 'control panel' stamp: 'Richo 7/16/2008 10:08'!
makeVideoServerControlPanel
	videoServerStringMorph := self boardGameVideoServer  asString asMorph.
	^ AlignmentMorph newRow color: Color transparent;
		 addMorphBack: (SimpleButtonMorph new label: '  VideoServer ...  ';
			
			color: (Color blue mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 borderWidth: 2;
			 actionSelector: #connectVideoServer;
			 target: self);
		 addMorphBack: (SimpleButtonMorph new label: '  Disconnect  ';
			
			color: (Color red mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 borderWidth: 2;
			 actionSelector: #disconnectVideoServer;
			 target: self);
		 addMorphBack: ' --> ' asMorph;
		 addMorphBack: videoServerStringMorph;
		 addMorphBack: AlignmentMorph newVariableTransparentSpacer! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/14/2008 10:18'!
pause

board pause! !

!RSViewerMorph methodsFor: 'errors' stamp: 'Richo 7/14/2008 12:08'!
robotServerError
	self inform: 'Robot server unselected'! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/14/2008 10:19'!
start
	self boardGameVideoServer
		ifNil: [^self videoServerError.
].
	self boardGameRobotServer
		ifNil: [^self robotServerError.
].
	self boardGameTeam
		ifNil: [^self teamError.
].
board start! !

!RSViewerMorph methodsFor: 'actions' stamp: 'Richo 7/12/2008 20:41'!
stop
board stop.
	self disconnectAll.
	self
		submorphsDo: [:m | m delete].
	self initializeSubmorphs! !

!RSViewerMorph methodsFor: 'errors' stamp: 'Richo 7/14/2008 12:08'!
teamError
	self inform: 'Team unselected'! !

!RSViewerMorph methodsFor: 'errors' stamp: 'Richo 7/14/2008 12:09'!
videoServerError
	self inform: 'Video server unselected'! !

BorderedMorph subclass: #RSRobotMorph
	instanceVariableNames: 'board myRobot teamColor playerColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Viewer'!

!RSRobotMorph commentStamp: 'Richo 6/22/2008 22:24' prior: 0!
"(RSRobotMorph equipo: Color orange color: Color red) openInWorld"

Representa grficamente un robot (nuestro o contrincante).
Guarda una referencia a la instancia de Robot (u OpponentRobot) que est representando (en la variable "miRobot") y es responsable de actualizar su propia posicin y rotacin de acuerdo a lo que "miRobot" indica.
Tambin responde al evento click abriendo un inspector de la instancia de Robot que representa, brindndonos informacin adicional durante el partido (como la posicin, rotacin, velocidad de las ruedas o Rol si se implementa).
!

!RSRobotMorph class methodsFor: 'instance creation' stamp: 'Richo 11/17/2008 15:14'!
teamColor: teamColor playerColor: playerColor 
	^ (self basicNew teamColor: teamColor playerColor: playerColor) initialize! !

!RSRobotMorph methodsFor: 'accessing' stamp: 'Richo 9/9/2007 18:59'!
board: anObject
	board := anObject! !

!RSRobotMorph methodsFor: 'accessing' stamp: 'Richo 9/9/2007 18:55'!
center
(self owner isMemberOf: TransformationMorph)
ifTrue:[^self owner center].
^super center
! !

!RSRobotMorph methodsFor: 'accessing' stamp: 'Richo 11/17/2008 15:29'!
center: aPoint 
"Para mantener las coordenadas correctas si el robot gira"
	(self owner isMemberOf: TransformationMorph)
		ifTrue: [^ self owner center: aPoint].
	^ super center: aPoint! !

!RSRobotMorph methodsFor: 'event handling' stamp: 'Richo 6/23/2008 09:41'!
click: evt 
	myRobot
		ifNil: [^ nil].
	myRobot inspect! !

!RSRobotMorph methodsFor: 'drawing' stamp: 'Richo 11/17/2008 15:40'!
drawOn: aCanvas 
	| centro |
	aCanvas fillRectangle: self bounds color: teamColor.
	centro := self width / 1.5.
	aCanvas
		drawPolygon: {self left @ (self bottom - centro). self left @ self bottom. self left + centro @ self bottom}
		color: playerColor
		borderWidth: 0
		borderColor: Color black.
	aCanvas
		drawPolygon: {self right @ (self top + centro). self right @ self top. self right - centro @ self top}
		color: Color gray
		borderWidth: 0
		borderColor: Color black.! !

!RSRobotMorph methodsFor: 'event handling' stamp: 'Richo 9/9/2007 20:46'!
handlesMouseDown: evt 
	^ true! !

!RSRobotMorph methodsFor: 'initialization' stamp: 'Richo 6/22/2008 22:10'!
initialize
	super initialize.
	self
		extent: (RSBoardMorph centimetresToPixels: 7.5 @ 7.5)! !

!RSRobotMorph methodsFor: 'stepping' stamp: 'Richo 6/23/2008 09:40'!
loadPosition: aPoint 
	"Posiciona el robot tomando como referencia el centro de la cancha"
	| diferenciaConElCentro difEnPixels posX posY posicionRobot |
	posicionRobot := aPoint.
	diferenciaConElCentro := posicionRobot - RSBoardMorph centerFieldInInches.
	difEnPixels := RSBoardMorph inchesToPixels: diferenciaConElCentro.
	"Agrego lo siguiente porque el eje de coordenadas de squeak tiene el 0@0
	arriba y el simulador lo tiene abajo"
	posX := board bounds center x + difEnPixels x.
	posY := board bounds center y - difEnPixels y.
	self center: posX @ posY! !

!RSRobotMorph methodsFor: 'stepping' stamp: 'Richo 6/23/2008 09:31'!
loadRotation: angle 

	self heading: (RSBoardMorph squeakAngle: angle truncated).! !

!RSRobotMorph methodsFor: 'event handling' stamp: 'Richo 9/9/2007 20:47'!
mouseDown: evt 
	evt hand waitForClicksOrDrag: self event: evt! !

!RSRobotMorph methodsFor: 'accessing' stamp: 'Richo 11/30/2007 22:45'!
myRobot: anObject 
	myRobot := anObject! !

!RSRobotMorph methodsFor: 'stepping' stamp: 'Richo 6/23/2008 09:41'!
step
	myRobot
		ifNil: [^ nil].
	self loadRotation: myRobot rotation degrees.
	self loadPosition: myRobot position! !

!RSRobotMorph methodsFor: 'stepping' stamp: 'Richo 6/23/2008 09:29'!
stepTime
	^ board stepTime! !

!RSRobotMorph methodsFor: 'private' stamp: 'Richo 11/17/2008 15:14'!
teamColor: aColor playerColor: anotherColor
teamColor := aColor.playerColor:=anotherColor! !

TestCase subclass: #RSAngleTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Tests'!

!RSAngleTest methodsFor: 'igual' stamp: 'Richo 6/21/2008 20:02'!
testIgual1
	| a1 a2 |
	a1 := RSAngle degrees: 45.
	a2 := RSAngle degrees: 45.
	self
		should: [a1 = a2].
	self
		shouldnt: [a1
				= (RSAngle degrees: 46)]! !

!RSAngleTest methodsFor: 'igual' stamp: 'Richo 6/21/2008 20:02'!
testIgual2
	| a1 a2 |
	a1 := RSAngle degrees: 45.
	a2 := RSAngle radians: 0.785398163397448.
	self
		should: [a1 = a2].
	self
		shouldnt: [a1
				= (RSAngle radians: 1)]! !

!RSAngleTest methodsFor: 'igual' stamp: 'Richo 6/21/2008 20:02'!
testIgual3
	| a1 a2 |
	a1 := RSAngle radians: 0.785398163397448.
	a2 := RSAngle degrees: 45.
	self
		should: [a1 = a2]! !

!RSAngleTest methodsFor: 'igual' stamp: 'Richo 6/21/2008 20:02'!
testIgual4
	| a1 a2 |
	a1 := RSAngle radians: 0.785398163397448.
	a2 := RSAngle radians: 0.78539817.
	self
		should: [a1 = a2]! !

!RSAngleTest methodsFor: 'igual' stamp: 'Richo 6/21/2008 20:02'!
testIgual5
	| a1 a2 |
	a1 := RSAngle degrees: 45.005.
	a2 := RSAngle degrees: 45.004.
	self
		should: [a1 = a2]! !

!RSAngleTest methodsFor: 'inicializacion' stamp: 'Richo 6/21/2008 20:04'!
testInicializacionGrados
	| a |
	a := RSAngle degrees: 45.
	self
		should: [a degrees = 45].
	self
		should: [a radians closeTo: 0.785398163397448].
! !

!RSAngleTest methodsFor: 'inicializacion' stamp: 'Richo 6/21/2008 20:05'!
testInicializacionGrados2
	| a |
	a := RSAngle degrees: -45.
	self
		should: [a degrees = -45].
	self
		should: [a radians closeTo: -0.785398163397448].
! !

!RSAngleTest methodsFor: 'inicializacion' stamp: 'Richo 6/21/2008 20:05'!
testInicializacionRadianes
	| a |
	a := RSAngle radians: 0.785398163397448.
	self
		should: [a degrees closeTo: 45].
	self
		should: [a radians closeTo: 0.785398163397448].
! !

!RSAngleTest methodsFor: 'inicializacion' stamp: 'Richo 6/21/2008 20:05'!
testInicializacionRadianes2
	| a |
	a := RSAngle radians: -0.785398163397448.
	self
		should: [a degrees closeTo: -45].
	self
		should: [a radians closeTo: -0.785398163397448].
! !

!RSAngleTest methodsFor: 'negated' stamp: 'Richo 6/21/2008 20:02'!
testNegated
	| a |
	a := RSAngle degrees: 45.
	self
		should: [a negated degrees closeTo: -45].
	self
		should: [a negated radians closeTo: -0.785398163397448]! !

!RSAngleTest methodsFor: 'negated' stamp: 'Richo 6/21/2008 20:02'!
testNegated2
	| a |
	a := RSAngle degrees: 180.
	self
		should: [a negated degrees closeTo: -180].
	self
		should: [a negated radians closeTo: -3.141592653589793]! !

!RSAngleTest methodsFor: 'opuesto' stamp: 'Richo 6/21/2008 20:03'!
testOpuesto
	| a |
	a := RSAngle degrees: 90.
	self
		should: [a opposite
				= (RSAngle degrees: -90)].
	self
		shouldnt: [a opposite
				= (RSAngle degrees: -91)]! !

!RSAngleTest methodsFor: 'opuesto' stamp: 'Richo 6/21/2008 20:03'!
testOpuesto2
	| a |
	a := RSAngle degrees: 45.
	self
		should: [a opposite
				= (RSAngle degrees: -135)].
	self
		shouldnt: [a opposite
				= (RSAngle degrees: -91)]! !

!RSAngleTest methodsFor: 'opuesto' stamp: 'Richo 6/21/2008 20:04'!
testOpuesto3
	| a |
	a := RSAngle degrees: 15.
	self
		should: [a opposite
				= (RSAngle degrees: -165)].
	self
		shouldnt: [a opposite
				= (RSAngle degrees: -91)]! !

!RSAngleTest methodsFor: 'opuesto' stamp: 'Richo 6/21/2008 20:04'!
testOpuesto4
	| a |
	a := RSAngle degrees: 0.
	self
		should: [a opposite
				= (RSAngle degrees: 180)].
	self
		shouldnt: [a opposite
				= (RSAngle degrees: -91)]! !

!RSAngleTest methodsFor: 'resta' stamp: 'Richo 6/21/2008 20:02'!
testResta1
	| a1 a2 |
	a1 := RSAngle degrees: 179.
	a2 := RSAngle degrees: -2.
	self
		should: [(a1 - a2) degrees = -179]! !

!RSAngleTest methodsFor: 'resta' stamp: 'Richo 6/21/2008 20:03'!
testResta2
	| a1 a2 |
	a1 := RSAngle degrees: 135.
	a2 := RSAngle degrees: -180.
	self
		should: [(a1 - a2) degrees = -45]! !

!RSAngleTest methodsFor: 'resta' stamp: 'Richo 6/21/2008 20:03'!
testResta3
	| a1 a2 |
	a1 := RSAngle degrees: -135.
	a2 := RSAngle degrees: -180.
	self
		should: [(a1 - a2) degrees = 45]! !

!RSAngleTest methodsFor: 'resta' stamp: 'Richo 6/21/2008 20:03'!
testResta4
	| a1 a2 |
	a1 := RSAngle degrees: -30.
	a2 := RSAngle degrees: 10.
	self
		should: [(a1 - a2) degrees = -40]! !

!RSAngleTest methodsFor: 'resta' stamp: 'Richo 6/21/2008 20:03'!
testResta5
	| a1 a2 |
	a1 := RSAngle degrees: 10.
	a2 := RSAngle degrees: -30.
	self
		should: [(a1 - a2) degrees = 40]! !

!RSAngleTest methodsFor: 'resta' stamp: 'Richo 6/21/2008 20:03'!
testResta6
	| a1 a2 |
	a1 := RSAngle degrees: -10.
	a2 := RSAngle degrees: 30.
	self
		should: [(a1 - a2) degrees = -40]! !

!RSAngleTest methodsFor: 'resta' stamp: 'Richo 6/21/2008 20:03'!
testResta7
	| a1 a2 |
	a1 := RSAngle degrees: -45.
	a2 := RSAngle degrees: 0.
	self
		should: [(a1 - a2) degrees = -45]! !

!RSAngleTest methodsFor: 'suma' stamp: 'Richo 6/21/2008 20:03'!
testSuma1
	| a1 a2 |
	a1 := RSAngle degrees: 45.
	a2 := RSAngle degrees: 90.
	self
		should: [(a1 + a2) degrees = 135]! !

!RSAngleTest methodsFor: 'suma' stamp: 'Richo 6/21/2008 20:03'!
testSuma2
	| a1 a2 |
	a1 := RSAngle degrees: 135.
	a2 := RSAngle degrees: 90.
	self
		should: [(a1 + a2) degrees = -135]! !

!RSAngleTest methodsFor: 'suma' stamp: 'Richo 6/21/2008 20:03'!
testSuma3
	| a1 a2 |
	a1 := RSAngle degrees: 45.
	a2 := RSAngle degrees: 180.
	self
		should: [(a1 + a2) degrees = -135]! !

!RSAngleTest methodsFor: 'suma' stamp: 'Richo 6/21/2008 20:03'!
testSuma4
	| a1 a2 |
	a1 := RSAngle degrees: -45.
	a2 := RSAngle degrees: 180.
	self
		should: [(a1 + a2) degrees = 135]! !

!RSAngleTest methodsFor: 'suma' stamp: 'Richo 6/21/2008 20:03'!
testSuma5
	| a1 a2 |
	a1 := RSAngle degrees: -45.
	a2 := RSAngle degrees: -180.
	self
		should: [(a1 + a2) degrees = 135]! !

!RSAngleTest methodsFor: 'suma' stamp: 'Richo 6/21/2008 20:03'!
testSuma6
	| a1 a2 |
	a1 := RSAngle degrees: -179.
	a2 := RSAngle degrees: -2.
	self
		should: [(a1 + a2) degrees = 179]! !

!RSAngleTest methodsFor: 'suma' stamp: 'Richo 6/21/2008 20:03'!
testSuma7
	| a1 a2 |
	a1 := RSAngle degrees: 0.
	a2 := RSAngle degrees: 45.
	self
		should: [(a1 + a2) degrees = 45]! !

TestCase subclass: #RSRobotExampleTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Tests'!

!RSRobotExampleTest methodsFor: 'calcularAngulo' stamp: 'Richo 6/22/2008 22:20'!
testCalcularAngulo
	| robot ang |
	robot := RSRobotExample new.
	ang := robot getAngle: 10 @ 10.
	self
		should: [ang degrees closeTo: 45]! !

!RSRobotExampleTest methodsFor: 'calcularAngulo' stamp: 'Richo 6/22/2008 22:20'!
testCalcularAngulo2
	| robot ang |
	robot := RSRobotExample new.
	ang := robot getAngle: 10 @ 0.
	self
		should: [ang degrees closeTo: 0]! !

!RSRobotExampleTest methodsFor: 'calcularAngulo' stamp: 'Richo 6/22/2008 22:20'!
testCalcularAngulo3
	| robot ang |
	robot := RSRobotExample new.
	ang := robot getAngle: 0 @ 10.
	self
		should: [ang degrees closeTo: 90]! !

!RSRobotExampleTest methodsFor: 'calcularAngulo' stamp: 'Richo 6/22/2008 22:20'!
testCalcularAngulo4
	| robot ang |
	robot := RSRobotExample new.
	ang := robot getAngle: -10 @ -10.
	self
		should: [ang degrees closeTo: -135]! !

!RSRobotExampleTest methodsFor: 'calcularAngulo' stamp: 'Richo 6/22/2008 22:20'!
testCalcularAngulo5
	| robot ang |
	robot := RSRobotExample new.
	ang := robot getAngle: -10 @ 10.
	self
		should: [ang degrees closeTo: 135]! !

!RSRobotExampleTest methodsFor: 'calcularAngulo' stamp: 'Richo 6/22/2008 22:20'!
testCalcularAngulo6
	| robot ang |
	robot := RSRobotExample new.
	ang := robot getAngle: 0 @ -10.
	self
		should: [ang degrees closeTo: -90]! !

!RSRobotExampleTest methodsFor: 'calcularAngulo' stamp: 'Richo 6/22/2008 22:20'!
testCalcularAngulo7
	| robot ang |
	robot := RSRobotExample new.
	ang := robot getAngle: -10 @ 0.
	self
		should: [ang degrees closeTo: 180]! !

!RSRobotExampleTest methodsFor: 'cuadranteDe' stamp: 'Richo 6/22/2008 22:20'!
testCuadranteDe
	| robot cuadrante |
	robot := RSRobotExample new.
	robot position: 0 @ 0.
	cuadrante := robot quadrant: 10 @ 10.
	self
		should: [cuadrante = 1].
	cuadrante := robot quadrant: -10 @ 10.
	self
		should: [cuadrante = 2].
	cuadrante := robot quadrant: -10 @ -10.
	self
		should: [cuadrante = 3].
	cuadrante := robot quadrant: 10 @ -10.
	self
		should: [cuadrante = 4]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:20'!
testEstaDelLado
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 45).
	robot position: 0 @ 0.
	obj := 10 @ 0.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		should: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:20'!
testEstaDelLado10
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 0).
	robot position: 0 @ 0.
	obj := 0 @ 10.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		shouldnt: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:20'!
testEstaDelLado11
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 90).
	robot position: 0 @ 0.
	obj := 10 @ -10.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		should: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:20'!
testEstaDelLado12
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 135).
	robot position: 0 @ 0.
	obj := 10 @ -1.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		should: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:20'!
testEstaDelLado13
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: -135).
	robot position: 0 @ 0.
	obj := 10 @ -1.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		shouldnt: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:20'!
testEstaDelLado14
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 0).
	robot position: 0 @ 0.
	obj := 0 @ 10.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		shouldnt: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:20'!
testEstaDelLado15
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 0).
	robot position: 0 @ 0.
	obj := 0 @ -10.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		should: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:20'!
testEstaDelLado2
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 45).
	robot position: 0 @ 0.
	obj := -10 @ 0.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		shouldnt: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:21'!
testEstaDelLado3
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 90).
	robot position: 0 @ 0.
	obj := -10 @ 0.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		shouldnt: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:21'!
testEstaDelLado4
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 90).
	robot position: 0 @ 0.
	obj := 10 @ 0.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		should: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:21'!
testEstaDelLado5
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: -90).
	robot position: 0 @ 0.
	obj := -10 @ 0.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		should: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:21'!
testEstaDelLado6
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 45).
	robot position: 0 @ 0.
	obj := 0 @ 10.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		shouldnt: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:21'!
testEstaDelLado7
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: -90).
	robot position: 0 @ 0.
	obj := 10 @ 0.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		shouldnt: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:21'!
testEstaDelLado8
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: -45).
	robot position: 0 @ 0.
	obj := 0 @ -10.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		should: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'estaDelLado' stamp: 'Richo 6/22/2008 22:21'!
testEstaDelLado9
	| robot obj |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 135).
	robot position: 0 @ 0.
	obj := -10 @ 0.
	self
		shouldnt: [robot isLookingTo: obj].
	self
		shouldnt: [robot isInTheRightSide: obj]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoA
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 45).
	robot position: 0 @ 0.
	self
		should: [robot isLookingTo: 10 @ 10].
	self
		shouldnt: [robot lookInFrontOf: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoA10
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 179).
	robot position: 0 @ 0.
	self
		should: [robot isLookingTo: -10 @ 0].
	self
		shouldnt: [robot lookInFrontOf: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoA2
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 1).
	robot position: 0 @ 0.
	self
		should: [robot isLookingTo: 10 @ 0].
	self
		shouldnt: [robot lookInFrontOf: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoA3
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 1).
	robot position: 0 @ 0.
	self
		should: [robot isLookingTo: 10 @ 0].
	self
		shouldnt: [robot lookInFrontOf: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoA4
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 90).
	robot position: 0 @ 0.
	self
		should: [robot isLookingTo: 0 @ -10].
	self
		should: [robot isLookingTo: 0 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoDeEspaldasA1
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 179).
	robot position: 0 @ 0.
	self
		should: [robot lookBackwardsTo: 10 @ 0].
	self
		shouldnt: [robot lookBackwardsTo: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoDeEspaldasA2
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 1).
	robot position: 0 @ 0.
	self
		should: [robot lookBackwardsTo: -10 @ 0].
	self
		shouldnt: [robot lookBackwardsTo: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoDeEspaldasA3
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 179).
	robot position: 0 @ 0.
	self
		should: [robot lookBackwardsTo: 10 @ 0].
	self
		shouldnt: [robot lookBackwardsTo: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoDeEspaldasA4
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 179).
	robot position: 0 @ 0.
	self
		should: [robot lookBackwardsTo: 10 @ 0].
	self
		shouldnt: [robot lookBackwardsTo: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoDeEspaldasA5
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 1).
	robot position: 0 @ 0.
	self
		should: [robot lookBackwardsTo: -10 @ 0].
	self
		shouldnt: [robot lookBackwardsTo: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoDeFrenteA
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 45).
	robot position: 0 @ 0.
	self
		should: [robot lookInFrontOf: 10 @ 10].
	self
		shouldnt: [robot lookInFrontOf: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoDeFrenteA10
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 179).
	robot position: 0 @ 0.
	self
		should: [robot lookInFrontOf: -10 @ 0].
	self
		shouldnt: [robot lookInFrontOf: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoDeFrenteA2
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 1).
	robot position: 0 @ 0.
	self
		should: [robot lookInFrontOf: 10 @ 0].
	self
		shouldnt: [robot lookInFrontOf: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:21'!
testMirandoDeFrenteA3
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 1).
	robot position: 0 @ 0.
	self
		should: [robot lookInFrontOf: 10 @ 0].
	self
		shouldnt: [robot lookInFrontOf: -10 @ 10]! !

!RSRobotExampleTest methodsFor: 'mirandoA' stamp: 'Richo 6/22/2008 22:22'!
testMirandoDeFrenteA4
	| robot |
	robot := RSRobotExample new.
	robot
		rotation: (RSAngle degrees: 90).
	robot position: 0 @ 0.
	self
		shouldnt: [robot lookInFrontOf: 0 @ -10].
	self
		should: [robot lookInFrontOf: 0 @ 10]! !

TestCase subclass: #RSViewerMorphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Tests'!

!RSViewerMorphTest methodsFor: 'as yet unclassified' stamp: 'Richo 6/22/2008 22:10'!
testAngleConversion
	| angSimu angSqueak |
	angSimu := 90.
	angSqueak := RSBoardMorph squeakAngle: angSimu.
	self
		should: [angSqueak = 0]! !

!RSViewerMorphTest methodsFor: 'as yet unclassified' stamp: 'Richo 6/22/2008 22:10'!
testAngleConversion2
	| angSimu angSqueak |
	angSimu := 0.
	angSqueak := RSBoardMorph squeakAngle: angSimu.
	self
		should: [angSqueak = 90]! !

!RSViewerMorphTest methodsFor: 'as yet unclassified' stamp: 'Richo 6/22/2008 22:10'!
testAngleConversion3
	| angSimu angSqueak |
	angSimu := -179.
	angSqueak := RSBoardMorph squeakAngle: angSimu.
	self
		should: [angSqueak = -91]! !

!RSViewerMorphTest methodsFor: 'as yet unclassified' stamp: 'Richo 6/22/2008 22:10'!
testAngleConversion4
	| angSimu angSqueak |
	angSimu := 179.
	angSqueak := RSBoardMorph squeakAngle: angSimu.
	self
		should: [angSqueak = -89]! !

Object subclass: #RSAngle
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Strategy'!

!RSAngle class methodsFor: 'instance creation' stamp: 'Richo 6/21/2008 19:57'!
degrees: aNumber 
	^ self basicNew inDegrees: aNumber! !

!RSAngle class methodsFor: 'instance creation' stamp: 'Richo 6/23/2008 11:49'!
new
self shouldNotImplement! !

!RSAngle class methodsFor: 'instance creation' stamp: 'Richo 6/21/2008 19:57'!
radians: aNumber
^self basicNew inRadians: aNumber! !

!RSAngle methodsFor: 'arithmetic' stamp: 'Richo 6/22/2008 22:15'!
+ anAngle 
	"Suma manteniendo entre -180 y 180"
	| nuevo |
	nuevo := self degrees + anAngle degrees.
	nuevo > 180
		ifTrue: [nuevo := self degrees - (360 - anAngle degrees)].
	nuevo < -180
		ifTrue: [^ self - anAngle negated].
	^ RSAngle degrees: nuevo! !

!RSAngle methodsFor: 'arithmetic' stamp: 'Richo 6/22/2008 22:15'!
- anAngle 
	"Resta manteniendo el valor entre -180 y 180"
	| nuevo |
	nuevo := self degrees - anAngle degrees.
	nuevo < -180
		ifTrue: [nuevo := self degrees + (360 - anAngle degrees)].
	nuevo > 180
		ifTrue: [^ self + anAngle negated].
	^ RSAngle degrees: nuevo! !

!RSAngle methodsFor: 'arithmetic' stamp: 'Richo 6/21/2008 20:01'!
< anAngle 
	^ self degrees < anAngle degrees! !

!RSAngle methodsFor: 'arithmetic' stamp: 'Richo 6/21/2008 20:01'!
= anAngle 
	^ self radians closeTo: anAngle radians! !

!RSAngle methodsFor: 'arithmetic' stamp: 'Richo 6/21/2008 20:01'!
> anAngle 
	^ self degrees > anAngle degrees! !

!RSAngle methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:59'!
degrees
^value radiansToDegrees ! !

!RSAngle methodsFor: 'arithmetic' stamp: 'Richo 6/21/2008 20:01'!
diff: anAngle 
	| dif |
	dif := self - anAngle.
	^ dif degrees abs! !

!RSAngle methodsFor: 'initialization' stamp: 'Richo 6/21/2008 19:59'!
inDegrees: aNumber 
	value := aNumber degreesToRadians ! !

!RSAngle methodsFor: 'initialization' stamp: 'Richo 6/21/2008 19:58'!
inRadians: aNumber
value := aNumber! !

!RSAngle methodsFor: 'arithmetic' stamp: 'Richo 6/22/2008 22:15'!
negated
	^ RSAngle degrees: self degrees negated! !

!RSAngle methodsFor: 'arithmetic' stamp: 'Richo 6/22/2008 22:15'!
opposite
	^ self
		+ (RSAngle degrees: 180)! !

!RSAngle methodsFor: 'printing' stamp: 'Richo 6/23/2008 11:47'!
printOn: aStream 
	super printOn: aStream.
	aStream nextPutAll: '(' , self degrees asString , ' degrees)'! !

!RSAngle methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:59'!
radians
	^ value! !

Object subclass: #RSBall
	instanceVariableNames: 'position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Strategy'!

!RSBall methodsFor: 'initialization' stamp: 'Richo 6/21/2008 19:47'!
initialize
position := 0@0! !

!RSBall methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:46'!
position
^position! !

!RSBall methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:51'!
position: aPoint 
position := aPoint! !

Object subclass: #RSEnvironment
	instanceVariableNames: 'home opponent currentBall lastBall fieldBounds goalBounds gameState whosBall isYellowTeam'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Strategy'!

!RSEnvironment class methodsFor: 'instance creation' stamp: 'Richo 7/3/2008 11:39'!
new
self shouldNotImplement! !

!RSEnvironment class methodsFor: 'instance creation' stamp: 'Richo 7/12/2008 21:49'!
numberOfRobots: aNumber teamClass: aTeamClass 
	^ self basicNew 
		initializeForTeam: aTeamClass
		withNumberOfRobots: aNumber; initialize! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
currentBall
	"Answer the value of currentBall"

	^ currentBall! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
currentBall: anObject
	"Set the value of currentBall"

	currentBall := anObject! !

!RSEnvironment methodsFor: 'inverting field' stamp: 'Richo 6/22/2008 18:03'!
decideTeam
	| rightSide leftSide |
	isYellowTeam
		ifNotNil: [^ self].
	rightSide := 0.
	leftSide := 0.
	home
		do: [:each | each position > fieldBounds center
				ifTrue: [rightSide := rightSide + 1]
				ifFalse: [leftSide := leftSide + 1]].
	opponent
		do: [:each | each position > fieldBounds center
				ifTrue: [rightSide := rightSide - 1]
				ifFalse: [leftSide := leftSide - 1]].
	isYellowTeam := leftSide > rightSide! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
fieldBounds
	"Answer the value of fieldBounds"

	^ fieldBounds! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
fieldBounds: anObject
	"Set the value of fieldBounds"

	fieldBounds := anObject! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:54'!
gameState
	"Answer the value of gameState"

	^ gameState! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:54'!
gameState: anObject
	"Set the value of gameState"

	gameState := anObject! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
goalBounds
	"Answer the value of goalBounds"

	^ goalBounds! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
goalBounds: anObject
	"Set the value of goalBounds"

	goalBounds := anObject! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
home
	"Answer the value of home"

	^ home! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
home: anObject
	"Set the value of home"

	home := anObject! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 20:22'!
homeAt: aSmallInteger
^home at: aSmallInteger ! !

!RSEnvironment methodsFor: 'initialize-release' stamp: 'Richo 7/3/2008 11:17'!
initializeForTeam: aTeamClass withNumberOfRobots: aNumber 
	home := Array new: aNumber.
	opponent := Array new: aNumber.
	1
		to: aNumber
		do: [:i | 
			home
				at: i
				put: (aTeamClass myRobot newWithEnvironment: self).
			opponent at: i put: aTeamClass myOpponent new].
	currentBall := aTeamClass myBall new.
	lastBall := aTeamClass myBall new! !

!RSEnvironment methodsFor: 'inverting field' stamp: 'Richo 6/22/2008 17:54'!
invertField
	self invertPosition: self currentBall.
	self invertPosition: self lastBall.
	self home
		do: [:r | 
			self invertPosition: r.
			self invertRotation: r].
	self opponent
		do: [:r | 
			self invertPosition: r.
			self invertRotation: r]! !

!RSEnvironment methodsFor: 'inverting field' stamp: 'Richo 6/22/2008 18:00'!
invertFieldIfYellowTeam
self decideTeam.
	isYellowTeam
		ifTrue: [self invertField]! !

!RSEnvironment methodsFor: 'inverting field' stamp: 'Richo 6/22/2008 21:38'!
invertPosition: aRobotOrBall 

	aRobotOrBall position: fieldBounds left + (fieldBounds right - aRobotOrBall position x) @ (fieldBounds top + (fieldBounds bottom - aRobotOrBall position y))! !

!RSEnvironment methodsFor: 'inverting field' stamp: 'Richo 6/22/2008 17:55'!
invertRotation: aRobot 
	"Suma 180 pero lo mantiene entre -180 y 180"
	aRobot rotation: aRobot rotation opposite! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/22/2008 17:49'!
isYellowTeam
^isYellowTeam ! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/22/2008 17:49'!
isYellowTeam: aBoolean
isYellowTeam := aBoolean! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
lastBall
	"Answer the value of lastBall"

	^ lastBall! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
lastBall: anObject
	"Set the value of lastBall"

	lastBall := anObject! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
opponent
	"Answer the value of opponent"

	^ opponent! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 18:45'!
opponent: anObject
	"Set the value of opponent"

	opponent := anObject! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 20:22'!
opponentAt: aSmallInteger

	^opponent at: aSmallInteger ! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:54'!
whosBall
	"Answer the value of whosBall"

	^ whosBall! !

!RSEnvironment methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:54'!
whosBall: anObject
	"Set the value of whosBall"

	whosBall := anObject! !

Object subclass: #RSGame
	instanceVariableNames: 'numberOfRobots videoServer robotServer rsEnvironment team process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Strategy'!

!RSGame class methodsFor: 'constants' stamp: 'Richo 7/14/2008 11:47'!
numberOfRobotsDefaultValue
^5! !

!RSGame class methodsFor: 'as yet unclassified' stamp: 'Richo 7/14/2008 11:46'!
videoServer: aVideoServer robotServer: aRobotServer team: aTeam 
	^ self basicNew videoServer: aVideoServer;
		 robotServer: aRobotServer;
		 team: aTeam; initialize! !

!RSGame methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:46'!
disconnectAll

	self disconnectVideoServer.
	self disconnectRobotServer.
	self disconnectTeam! !

!RSGame methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:47'!
disconnectRobotServer
	self stop.
	robotServer
		ifNotNil: [robotServer disconnect.
			robotServer := nil]! !

!RSGame methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:47'!
disconnectTeam
	self stop.
	team
		ifNotNil: [team := nil]! !

!RSGame methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:47'!
disconnectVideoServer
	self stop.
	videoServer
		ifNotNil: [videoServer disconnect.
			videoServer := nil]! !

!RSGame methodsFor: 'initialize' stamp: 'Richo 7/14/2008 11:46'!
initialize
	numberOfRobots := self class numberOfRobotsDefaultValue! !

!RSGame methodsFor: 'accessing' stamp: 'Richo 7/14/2008 11:44'!
initializeEnvironment
	rsEnvironment := team class myEnvironment numberOfRobots: self numberOfRobots teamClass: team class! !

!RSGame methodsFor: 'accessing' stamp: 'Richo 7/14/2008 11:45'!
numberOfRobots
	^ numberOfRobots
! !

!RSGame methodsFor: 'accessing' stamp: 'Richo 7/14/2008 11:44'!
numberOfRobots: aSmallInteger
numberOfRobots:=aSmallInteger ! !

!RSGame methodsFor: 'actions' stamp: 'Richo 7/12/2008 21:39'!
pause
process ifNotNil: [
	process suspend]! !

!RSGame methodsFor: 'accessing' stamp: 'Richo 7/12/2008 21:41'!
robotServer
	"Answer the value of robotServer"

	^ robotServer! !

!RSGame methodsFor: 'accessing' stamp: 'Richo 7/12/2008 21:41'!
robotServer: anObject
	"Set the value of robotServer"

	robotServer := anObject! !

!RSGame methodsFor: 'accessing' stamp: 'Richo 7/12/2008 21:41'!
rsEnvironment
	"Answer the value of rsEnvironment"

	^ rsEnvironment! !

!RSGame methodsFor: 'actions' stamp: 'Richo 7/14/2008 10:09'!
start
	process
		ifNotNil: [process resume.
			^ self].
	process := [[true]
				whileTrue: [self step]]
				forkNamed: self class asString! !

!RSGame methodsFor: 'stepping' stamp: 'Richo 7/14/2008 10:10'!
step

	videoServer storeDataOn: rsEnvironment.
	rsEnvironment invertFieldIfYellowTeam.
	team strategyFor: rsEnvironment.
	robotServer sendMovesOf: rsEnvironment home! !

!RSGame methodsFor: 'actions' stamp: 'Richo 7/12/2008 22:34'!
stop
	process
		ifNotNil: [process terminate. process := nil]! !

!RSGame methodsFor: 'accessing' stamp: 'Richo 7/12/2008 21:41'!
team
	"Answer the value of team"

	^ team! !

!RSGame methodsFor: 'accessing' stamp: 'Richo 7/12/2008 22:09'!
team: aTeam 
"Carga el team y manda a inicializar el environment"
	team := aTeam.
	self initializeEnvironment! !

!RSGame methodsFor: 'accessing' stamp: 'Richo 7/12/2008 21:41'!
videoServer
	"Answer the value of videoServer"

	^ videoServer! !

!RSGame methodsFor: 'accessing' stamp: 'Richo 7/12/2008 21:41'!
videoServer: anObject
	"Set the value of videoServer"

	videoServer := anObject! !

Object subclass: #RSOpponentRobot
	instanceVariableNames: 'rotation position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Strategy'!

!RSOpponentRobot methodsFor: 'initialization' stamp: 'Richo 6/23/2008 11:50'!
initialize
	rotation := RSAngle degrees: 0.
	position := 0 @ 0! !

!RSOpponentRobot methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:52'!
position
	"Answer the value of position"

	^ position! !

!RSOpponentRobot methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:52'!
position: anObject
	"Set the value of position"

	position := anObject! !

!RSOpponentRobot methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:52'!
rotation
	"Answer the value of rotation"

	^ rotation! !

!RSOpponentRobot methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:52'!
rotation: anObject
	"Set the value of rotation"

	rotation := anObject! !

Object subclass: #RSRobot
	instanceVariableNames: 'rsEnvironment rotation position velocityLeft velocityRight'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Strategy'!

!RSRobot class methodsFor: 'instance creation' stamp: 'Richo 7/3/2008 11:17'!
newWithEnvironment: anEnvironment 
	^ (self basicNew setEnvironment: anEnvironment) initialize! !

!RSRobot methodsFor: 'accessing' stamp: 'Richo 7/3/2008 11:08'!
environment
	"Answer the value of environment"
	^ rsEnvironment! !

!RSRobot methodsFor: 'initialize-release' stamp: 'Richo 6/23/2008 11:50'!
initialize
	rotation := RSAngle degrees: 0.
	position := 0 @ 0.
	velocityLeft := 0.
	velocityRight := 0! !

!RSRobot methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:49'!
position
	"Answer the value of position"

	^ position! !

!RSRobot methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:49'!
position: anObject
	"Set the value of position"

	position := anObject! !

!RSRobot methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:49'!
rotation
	"Answer the value of rotation"

	^ rotation! !

!RSRobot methodsFor: 'accessing' stamp: 'Richo 7/5/2008 21:09'!
rotation: anObject 
	"Set the value of rotation"
rotation := anObject! !

!RSRobot methodsFor: 'private' stamp: 'Richo 7/3/2008 11:08'!
setEnvironment: anEnvironment 
	rsEnvironment := anEnvironment! !

!RSRobot methodsFor: 'printing' stamp: 'Richo 6/21/2008 19:49'!
velocityAsString	^ velocityLeft asString , ' ' , velocityRight asString! !

!RSRobot methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:49'!
velocityLeft
	"Answer the value of velocityLeft"

	^ velocityLeft! !

!RSRobot methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:49'!
velocityLeft: anObject
	"Set the value of velocityLeft"

	velocityLeft := anObject! !

!RSRobot methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:49'!
velocityRight
	"Answer the value of velocityRight"

	^ velocityRight! !

!RSRobot methodsFor: 'accessing' stamp: 'Richo 6/21/2008 19:49'!
velocityRight: anObject
	"Set the value of velocityRight"

	velocityRight := anObject! !

RSRobot subclass: #RSRobotExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Strategy-Example'!

!RSRobotExample methodsFor: 'goalkeeper' stamp: 'Richo 6/22/2008 21:59'!
beGoalkeeper
	(self position x - 90) abs > 5
		ifTrue: [^ self moveTo: 90 @ self position y].
	(self rotation
			diff: (RSAngle degrees: -90))
			> 5
		ifTrue: [^ self lookDown].
	self followBallInY! !

!RSRobotExample methodsFor: 'goalkeeper' stamp: 'Richo 6/22/2008 22:03'!
followBallInY
	| pelotaY dif speed |
	pelotaY := self environment currentBall position y.
	dif := pelotaY - self position y.
	speed := 125.
	pelotaY < 50 & (pelotaY > 30)
		ifTrue: [dif > 1
				ifTrue: ["Esta arriba"
					^ self velocityLeft: speed negated;
						 velocityRight: speed negated].
			dif < -1
				ifTrue: ["Esta abajo"
					^ self velocityLeft: speed;
						 velocityRight: speed]].
	^ self velocityLeft: 0;
		 velocityRight: 0! !

!RSRobotExample methodsFor: 'calculos' stamp: 'Richo 6/22/2008 21:59'!
getAngle: aPoint 
	| ang |
	(self quadrant: aPoint)
			= 1
		ifTrue: [ang := self getAngleInQuadrant1: aPoint].
	(self quadrant: aPoint)
			= 2
		ifTrue: [ang := self getAngleInQuadrant2: aPoint].
	(self quadrant: aPoint)
			= 3
		ifTrue: [ang := self getAngleInQuadrant3: aPoint].
	(self quadrant: aPoint)
			= 4
		ifTrue: [ang := self getAngleInQuadrant4: aPoint].
	^ RSAngle degrees: ang! !

!RSRobotExample methodsFor: 'calculos' stamp: 'Richo 6/22/2008 21:57'!
getAngleInQuadrant1: aPoint 
	| dx dy |
	dx := (aPoint x - self position x) abs.
	dy := (aPoint y - self position y) abs.
	dx = 0
		ifTrue: [^ 90.0].
	^ (dy / dx) arcTan radiansToDegrees! !

!RSRobotExample methodsFor: 'calculos' stamp: 'Richo 6/22/2008 21:57'!
getAngleInQuadrant2: aPoint 
	^ 180
		- (self getAngleInQuadrant1: aPoint)! !

!RSRobotExample methodsFor: 'calculos' stamp: 'Richo 6/22/2008 21:57'!
getAngleInQuadrant3: aPoint 
	^ (self getAngleInQuadrant2: aPoint) negated! !

!RSRobotExample methodsFor: 'calculos' stamp: 'Richo 6/22/2008 21:57'!
getAngleInQuadrant4: aPoint 
	^ (self getAngleInQuadrant1: aPoint) negated! !

!RSRobotExample methodsFor: 'calculos' stamp: 'Richo 6/22/2008 21:57'!
isInTheRightSide: aPoint 
	| miAng miOpuesto ang |
	miAng := self rotation.
	miOpuesto := miAng opposite.
	ang := self getAngle: aPoint.
	miAng degrees > 0
		ifTrue: [^ (ang > miAng | (ang < miOpuesto)) not]
		ifFalse: [^ ang < miAng | (ang > miOpuesto)]! !

!RSRobotExample methodsFor: 'calculos' stamp: 'Richo 6/22/2008 21:57'!
isLookingTo: aPoint 
	^ (self lookInFrontOf: aPoint)
		| (self lookBackwardsTo: aPoint)! !

!RSRobotExample methodsFor: 'calculos' stamp: 'Richo 6/22/2008 21:57'!
lookBackwardsTo: aPoint 
	| angulo |
	angulo := self getAngle: aPoint.
	^ (self rotation opposite diff: angulo)
		< 20! !

!RSRobotExample methodsFor: 'goalkeeper' stamp: 'Richo 6/22/2008 21:58'!
lookDown
	| direccion speed |
	direccion := self position x @ (self position y - 20).
	speed := 10.
	(self isInTheRightSide: direccion)
		ifTrue: [self velocityLeft: speed;
				 velocityRight: speed negated]
		ifFalse: [self velocityLeft: speed negated;
				 velocityRight: speed]! !

!RSRobotExample methodsFor: 'calculos' stamp: 'Richo 6/22/2008 21:58'!
lookInFrontOf: aPoint 
	| angulo |
	angulo := self getAngle: aPoint.
	^ (self rotation diff: angulo)
		< 20! !

!RSRobotExample methodsFor: 'navigation' stamp: 'Richo 6/22/2008 21:58'!
moveTo: aPoint 
	(self lookInFrontOf: aPoint)
		ifTrue: [^ self velocityLeft: 125;
				 velocityRight: 125].
	(self lookBackwardsTo: aPoint)
		ifTrue: [^ self velocityLeft: -125;
				 velocityRight: -125].
	(self isInTheRightSide: aPoint)
		ifTrue: [self velocityLeft: 125;
				 velocityRight: 50]
		ifFalse: [self velocityLeft: 50;
				 velocityRight: 125]! !

!RSRobotExample methodsFor: 'navigation' stamp: 'Richo 6/22/2008 22:01'!
moveToBall
	| posPelota |
	posPelota := self environment currentBall position.
	self position x > posPelota x
		ifTrue: [self moveTo: posPelota]
		ifFalse: [self position y < posPelota y
				ifTrue: [self moveTo: posPelota x + 10 @ (posPelota y - 10)]
				ifFalse: [self moveTo: posPelota x + 10 @ (posPelota y + 10)]]! !

!RSRobotExample methodsFor: 'calculos' stamp: 'Richo 6/22/2008 21:58'!
quadrant: aPoint 
	aPoint x >= self position x
		ifTrue: [aPoint y >= self position y
				ifTrue: [^ 1]
				ifFalse: [^ 4]]
		ifFalse: [aPoint y >= self position y
				ifTrue: [^ 2]
				ifFalse: [^ 3]]! !

Object subclass: #RSRobotServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Servers'!

!RSRobotServer class methodsFor: 'instance creation' stamp: 'Richo 7/14/2008 12:48'!
fromUser
	"Pide los datos necesarios al usuario y crea el objeto cargando los datos
	necesarios"
	^ self subclassResponsibility! !

!RSRobotServer methodsFor: 'as yet unclassified' stamp: 'Richo 6/23/2008 11:05'!
sendMovesOf: aCollectionOfRobots
	"Enva la velocidad de las ruedas de todos los robots"
	self subclassResponsibility! !

RSRobotServer subclass: #RSSimulatedRobotServer
	instanceVariableNames: 'socket ip port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Servers-Simulated'!

RSSimulatedRobotServer class
	instanceVariableNames: 'lastIp lastPort'!

RSSimulatedRobotServer class
	instanceVariableNames: 'lastIp lastPort'!

!RSSimulatedRobotServer class methodsFor: 'instance creation' stamp: 'Richo 7/9/2008 22:49'!
fromUser
	| ip port |
	ip := FillInTheBlank request: 'IP:' initialAnswer: self lastIp.
	port := (FillInTheBlank request: 'Port:' initialAnswer: self lastPort asString) asInteger.
	^ self ip: ip port: port! !

!RSSimulatedRobotServer class methodsFor: 'instance creation' stamp: 'Richo 7/9/2008 22:49'!
ip: aString port: aSmallInteger 
lastIp := aString.
lastPort := aSmallInteger
	.^ (self basicNew setIp: aString setPort: aSmallInteger) initialize! !

!RSSimulatedRobotServer class methodsFor: 'instance creation' stamp: 'Richo 7/9/2008 22:49'!
lastIp
^lastIp ifNil: ['localhost']! !

!RSSimulatedRobotServer class methodsFor: 'instance creation' stamp: 'Richo 7/9/2008 22:49'!
lastPort
^lastPort ifNil: [6363]! !

!RSSimulatedRobotServer methodsFor: 'initialize-release' stamp: 'Richo 6/22/2008 17:44'!
disconnect

	socket closeAndDestroy! !

!RSSimulatedRobotServer methodsFor: 'initialize-release' stamp: 'Richo 6/23/2008 11:07'!
initialize
	socket := Socket newUDP.! !

!RSSimulatedRobotServer methodsFor: 'printing' stamp: 'Richo 7/12/2008 23:28'!
printOn: aStream 
	super printOn: aStream.
	aStream nextPutAll: '(' , ip , ', ' , port asString , ')'! !

!RSSimulatedRobotServer methodsFor: 'sending' stamp: 'Richo 6/26/2008 11:07'!
sendMovesOf: aCollectionOfRobots 
	| str |

	str := ''.
	aCollectionOfRobots
		do: [:r | str := str , r velocityAsString , ' '].
	socket
		sendData: str
		toHost: (NetNameResolver addressForName: ip)
		port: port! !

!RSSimulatedRobotServer methodsFor: 'private' stamp: 'Richo 6/21/2008 19:09'!
setIp: aString setPort: aSmallInteger 
ip:= aString.
port := aSmallInteger ! !

Object subclass: #RSTeam
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Strategy'!

!RSTeam class methodsFor: 'factory' stamp: 'Richo 6/23/2008 11:01'!
myBall
^RSBall! !

!RSTeam class methodsFor: 'factory' stamp: 'Richo 6/23/2008 11:20'!
myEnvironment
^RSEnvironment! !

!RSTeam class methodsFor: 'factory' stamp: 'Richo 6/23/2008 11:02'!
myOpponent
	^ RSOpponentRobot! !

!RSTeam class methodsFor: 'factory' stamp: 'Richo 7/3/2008 10:58'!
myRobot
	^ RSRobot! !

!RSTeam methodsFor: 'strategy' stamp: 'Richo 7/12/2008 21:43'!
strategyFor: anEnvironment
	self subclassResponsibility! !

RSTeam subclass: #RSTeamExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Strategy-Example'!

!RSTeamExample class methodsFor: 'factory' stamp: 'Richo 6/22/2008 21:56'!
myRobot
^RSRobotExample! !

!RSTeamExample methodsFor: 'strategy' stamp: 'Richo 7/12/2008 21:44'!
strategyFor: env
env home
		do: [:r | r moveToBall].
	(env homeAt: 1) beGoalkeeper! !

Object subclass: #RSVideoServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Servers'!

RSVideoServer subclass: #RSSimulatedVideoServer
	instanceVariableNames: 'socket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Servers-Simulated'!

RSSimulatedVideoServer class
	instanceVariableNames: 'lastPort'!

RSSimulatedVideoServer class
	instanceVariableNames: 'lastPort'!

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:27'!
anyonesBall
	^ 0! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:27'!
blueBall
	^ 1! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:27'!
fBot
	^ 6.373! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
fLeftX
	^ 6.8118! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
fRightX
	^ 93.4259! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
fTop
	^ 77.2392! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
freeBall
	^ 1! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
freeKick
	^ 4! !

!RSSimulatedVideoServer class methodsFor: 'instance creation' stamp: 'Richo 7/14/2008 10:22'!
fromUser
	| port |
	port := (FillInTheBlank request: 'Port:' initialAnswer: self lastPort asString) asInteger.

	^ self port: port! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
gBotY
	^ 33.932! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
gLeft
	^ 2.8748! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
gRight
	^ 97.3632! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
gTopY
	^ 49.6801! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
goalKick
	^ 5! !

!RSSimulatedVideoServer class methodsFor: 'instance creation' stamp: 'Richo 7/9/2008 22:47'!
lastPort
^lastPort ifNil: [6364]! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
penaltyKick
	^ 3! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:28'!
placeKick
	^ 2! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:29'!
playersPerSide
	^ 5! !

!RSSimulatedVideoServer class methodsFor: 'instance creation' stamp: 'Richo 7/9/2008 22:48'!
port: aSmallInteger 
lastPort := aSmallInteger.
	^ self basicNew
		setSocket: (Socket newUDP setPort: aSmallInteger)! !

!RSSimulatedVideoServer class methodsFor: 'constants' stamp: 'Richo 6/21/2008 19:29'!
yellowBall
	^ 2! !

!RSSimulatedVideoServer methodsFor: 'initialize-release' stamp: 'Richo 6/21/2008 18:58'!
disconnect
	socket closeAndDestroy! !

!RSSimulatedVideoServer methodsFor: 'receiving' stamp: 'Richo 6/21/2008 18:43'!
getData
	| data |
	data := String new: 512.
	socket waitForData; receiveDataInto: data.
	^data! !

!RSSimulatedVideoServer methodsFor: 'printing' stamp: 'Richo 7/12/2008 23:31'!
printOn: aStream 
	super printOn: aStream.
	aStream nextPutAll: '(' , socket port asString , ')'! !

!RSSimulatedVideoServer methodsFor: 'private' stamp: 'Richo 6/21/2008 19:06'!
setSocket: aSocket 
socket := aSocket! !

!RSSimulatedVideoServer methodsFor: 'parsing' stamp: 'Richo 7/14/2008 12:24'!
storeDataOn: env 
	| data parser variable resultado i |
	data := self getData.
	parser := TextDiffBuilder new.
	variable := parser split: data by: Character lf.
	resultado := variable
				collect: [:each | parser split: each by: Character space].
	env lastBall position: env currentBall position x @ env currentBall position y.
	env currentBall position: ((resultado at: 1)
			at: 1) asNumber @ ((resultado at: 1)
				at: 2) asNumber.
	i := 1.
	env home
		do: [:robot | 
			robot position: ((resultado at: i + 1)
					at: 1) asNumber @ ((resultado at: i + 1)
						at: 2) asNumber.
			robot
				rotation: (RSAngle degrees: ((resultado at: i + 1)
							at: 4) asNumber).
			i := i + 1].
	i := 1.
	env opponent
		do: [:opponent | 
			opponent position: ((resultado at: i + 6)
					at: 1) asNumber @ ((resultado at: i + 6)
						at: 2) asNumber.
			opponent
				rotation: (RSAngle degrees: ((resultado at: i + 6)
							at: 4) asNumber).
			i := i + 1].
	env gameState: ((resultado at: 12)
			at: 1) asNumber.
	env whosBall: ((resultado at: 12)
			at: 2) asNumber.
	env
		fieldBounds: (self class fLeftX @ self class fTop corner: self class fRightX @ self class fBot).
	env
		goalBounds: (self class gLeft @ self class gTopY corner: self class gRight @ self class gBotY)! !

!RSVideoServer class methodsFor: 'instance creation' stamp: 'Richo 6/22/2008 17:42'!
fromUser
"Devuelve la conexin al servidor de video"
	^ self subclassResponsibility! !

!RSVideoServer methodsFor: 'as yet unclassified' stamp: 'Richo 6/22/2008 17:53'!
storeDataOn: anEnvironment 
	"Recibe los datos, los parsea y los carga en el environment
	Datos que debera cargar en el environment:
	- home (Coleccin de RSRobot - Posicin y rotacin de cada robot
	propio). 
	- opponent (Coleccin de RSOpponentRobot - Posicin y rotacin
	de cada robot contrario).
	- currentBall (RSBall - Posicin de la pelota en este momento).
	- lastBall (RSBall - ltima posicin de la pelota).
	- fieldBounds (Rectangle - Lmites de la cancha).
	- goalBounts (Rectangle - Lmites de los arcos).
	- gameState (?).
	- whosBall (?).
	"
	self subclassResponsibility! !

Object subclass: #RSViewerPlugin
	instanceVariableNames: 'board isActive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RobotSoccer-Viewer'!

!RSViewerPlugin class methodsFor: 'as yet unclassified' stamp: 'Richo 7/7/2008 11:59'!
board: aRSBoardMorph 
	^ (self basicNew board: aRSBoardMorph) initialize! !

!RSViewerPlugin methodsFor: 'accessing' stamp: 'Richo 7/7/2008 11:59'!
board
	^board! !

!RSViewerPlugin methodsFor: 'accessing' stamp: 'Richo 7/7/2008 11:59'!
board: aRSBoardMorph 
	board := aRSBoardMorph! !

!RSViewerPlugin methodsFor: 'initialize-release' stamp: 'Richo 7/7/2008 12:29'!
initialize
isActive := false! !

!RSViewerPlugin methodsFor: 'accessing' stamp: 'Richo 7/7/2008 12:23'!
isActive
^isActive! !

!RSViewerPlugin methodsFor: 'accessing' stamp: 'Richo 7/7/2008 12:21'!
menu
	"Debe devolver un menu para activar/desactivar el plugin"
	self subclassResponsibility! !

!RSViewerPlugin methodsFor: 'stepping' stamp: 'Richo 7/7/2008 12:22'!
step
self subclassResponsibility! !

!RSViewerPlugin methodsFor: 'stepping' stamp: 'Richo 7/7/2008 12:24'!
stepIfActive
	isActive ifTrue: [self step]! !
